﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class GreaterThan : CodeNode, ILogicalComparator {
    public override int MinimumArity { get { return 2; } }
    public override int MaximumArity { get { return 2; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected GreaterThan(bool deserializing) : base(deserializing) { }
    protected GreaterThan(GreaterThan original, Cloner cloner) : base(original, cloner) { }

    public GreaterThan() : base("GreaterThan", "GreaterThan comparator.") { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GreaterThan(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      ISymbolicExpressionTreeNode lhs = null, rhs = null;
      var enumerator = children.GetEnumerator();
      int childCount = 0;
      while (enumerator.MoveNext()) {
        childCount++;
        switch (childCount) {
          case 1: lhs = enumerator.Current; break;
          case 2: rhs = enumerator.Current; break;
          default: throw new System.Exception("Unexpected number of children. Expected 2 children.");
        }
      }
      if (childCount < 2) throw new System.Exception("Unexpected number of children. Expected 2 children.");

      var result = new[] {
        ((CodeNode)lhs.Symbol).Interpret(lhs, lhs.Subtrees),
        ((CodeNode)rhs.Symbol).Interpret(rhs, rhs.Subtrees)
      };

      return result[0] + " > " + result[1];
    }
  }
}
