﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class Negation : CodeNode {
    public override int MinimumArity { get { return 1; } }
    public override int MaximumArity { get { return 1; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected Negation(bool deserializing) : base(deserializing) { }
    protected Negation(Negation original, Cloner cloner) : base(original, cloner) { }

    public Negation()
      : base("Negation", "A negation.") {
      Prefix = "!(";
      Suffix = ")";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Negation(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      var enumerator = children.GetEnumerator();
      if (!enumerator.MoveNext()) throw new System.Exception("Negation was not given a child.");

      var symbol = enumerator.Current.Symbol;
      if (!(symbol is LogicalValue || symbol is Conjunction || symbol is Disjunction || symbol is Negation || symbol is ILogicalComparator))
        throw new System.Exception("Negation was given a child of type " + symbol.GetType() +
            ". The expected child must be of type " + typeof(LogicalValue)
                                           + " or " + typeof(Conjunction)
                                           + " or " + typeof(Disjunction)
                                           + " or " + typeof(Negation)
                                           + " or " + typeof(ILogicalComparator) + ".");

      string result = ((CodeNode)symbol).Interpret(enumerator.Current, enumerator.Current.Subtrees);
      if (enumerator.MoveNext()) throw new System.Exception("LogicalExpression was given more than one child.");

      return Prefix + result + Suffix;
    }
  }
}
