﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class IfStat : CodeNode {
    public override int MinimumArity { get { return 2; } }
    public override int MaximumArity { get { return 3; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected IfStat(bool deserializing) : base(deserializing) { }
    protected IfStat(IfStat original, Cloner cloner) : base(original, cloner) { }

    public IfStat()
      : base("IfStat", "An if statement.") {
      Prefix = "if (";
      Suffix = ") {";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IfStat(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      string condition = string.Empty, ifTrue = string.Empty, ifElse = string.Empty;
      foreach (var c in children) {
        if (c.Symbol is LogicalExpression)
          condition = ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
        else if (c.Symbol is Stat)
          ifTrue = ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
        else if (c.Symbol is ElseStat || c.Symbol is EmptyEvent || c.Symbol is DoNothing)
          ifElse = ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
        else throw new ArgumentException("Unexpected children.", "children");
      }
      string result = Prefix + condition + Suffix + Environment.NewLine + ifTrue + Environment.NewLine + "}";
      result += string.IsNullOrEmpty(ifElse) ? Environment.NewLine : ifElse;
      return result;
    }
  }
}
